#!/usr/local/bin/guile \
-e gdc -s
Simple ncurses-gdc like clock. You can run it 
on different terminal line by "gdc.scm /dev/termname".
Quit it by pressing 'q'.
!#

(use-modules (ncurses ncurses))
(use-modules (ncurses util))

(debug-enable `backtrace)

(define xorigin 10)
(define yorigin 10)
(define fg_char #f)
(define bg_char #f)
(define digitattr #f)
(define boxattr #f)

(define (gdc-init-screen . tty)
  (if (null? tty)
	(ncurses-initscr)
	(ncurses-initscr (car tty)))
  (start-color)
  (noecho)
  (cbreak)
  (keypad (stdscr) #t)
  (init-pair 1 COLOR_BLUE COLOR_BLACK)
  (init-pair 2 COLOR_WHITE COLOR_BLACK)
  (curs-set 0)
  (halfdelay 2)
  (set! fg_char (ACS_BLOCK))
  (set! bg_char (char->integer #\ ))
  (set! digitattr (logior (COLOR-PAIR 1) A_BOLD))
  (set! boxattr (COLOR-PAIR 2))
  (set! xorigin (- (quotient (COLS) 2) 26))
  (set! yorigin (- (quotient (LINES) 2) 7)))

(define (gdc-done-screen)
  (standend)
  (clear)
  (refresh)
  (curs-set 1)
  (endwin))

(define (gdc args)
  (if (null? (cdr args))
    (gdc-init-screen)
    (gdc-init-screen (cadr args)))
  (gdc-loop)
  (gdc-done-screen))

(define digits `((".####."
			   	  "#....#"
				  "#....#"
				  "#....#"
				  "#....#"
				  "#....#"
				  "#....#"
				  ".####.") ;;; 0
				 
				 ("...#.."
			   	  "...#.."
				  "...#.."
				  "...#.."
				  "...#.."
				  "...#.."
				  "...#.."
				  "...#..") ;;; 1
				 
				 (".####."
				  "#....#"
				  "#....#"
				  "....#."
				  "...#.."
				  "..#..."
				  ".#...."
				  "######") ;;; 2
				 
				 (".####."
				  "#....#"
				  ".....#"
				  "....#."
				  "....#."
				  ".....#"
				  "#....#"
				  ".####.") ;;; 3
				 
				 ("#....#"
				  "#....#"
				  "#....#"
				  "#....#"
				  ".#####"
				  ".....#"
				  ".....#"
				  ".....#") ;;; 4
				 
				 ("######"
				  "#....."
				  "#....."
				  "#####."
				  ".....#"
				  ".....#"
				  "#....#"
				  ".####.") ;;; 5
				 
				 ("..####"
				  ".#...."
				  "#....."
				  "#####."
				  "#....#"
				  "#....#"
				  "#....#"
				  ".####.") ;;; 6
				 
				 ("######"
				  ".....#"
				  "....#."
				  "...#.."
				  "..#..."
				  "..#..."
				  "..#..."
				  "..#...") ;;; 7
				 
				 (".####."
				  "#....#"
				  "#....#"
				  ".####."
				  "#....#"
				  "#....#"
				  "#....#"
				  ".####.") ;;; 8
				 
				 (".####."
				  "#....#"
				  "#....#"
				  ".#####"
				  ".....#"
				  ".....#"
				  "#....#"
				  ".####."))) ;;; 9

;;; Main loop
(define (gdc-loop)
  (gdc-draw-box)
  (gdc-draw-digits 97871 0)
  (letrec ((time (current-time))
           (otime 97871) ;;; magic number
		   (loop (lambda ()
    	  	       (if (not (eq? time otime))
	  	    	     (begin
	  			       (gdc-draw-digits time otime)
	  			       (refresh)))
    	  		   (if (not (eq? (getch) (char->integer #\q)))
	  	    	     (begin
				       (set! otime time)
	  			       (set! time (current-time))
      			       (loop))))))
    (loop)))

(define (gdc-draw-digits time otime)
  (attrset digitattr)
  (let loopx ((x `(2 9 19 26 36 43))
  	   		  (dl (list (quotient (tm:hour (localtime time)) 10)
					(remainder (tm:hour (localtime time)) 10)
					(quotient (tm:min (localtime time)) 10)
					(remainder (tm:min (localtime time)) 10)
					(quotient (tm:sec (localtime time)) 10)
					(remainder (tm:sec (localtime time)) 10)))
			  (ol (list (quotient (tm:hour (localtime otime)) 10)
					(remainder (tm:hour (localtime otime)) 10)
					(quotient (tm:min (localtime otime)) 10)
					(remainder (tm:min (localtime otime)) 10)
					(quotient (tm:sec (localtime otime)) 10)
					(remainder (tm:sec (localtime otime)) 10))))
	(if (not (eq? (car dl) (car ol)))
      (let loopy ((y `(2 3 4 5 6 7 8 9))
		 	      (cdigit (list-ref digits (car dl))))
	    (letrec ((cline (string->list (car cdigit)))
	  		     (n 0)
	  	         (clr (lambda ()
				        (move (+ yorigin (car y)) (+ xorigin (car x) n))
					    (if (and (eq? (list-ref cline n) #\#)
					             (not (and (eq? 0 (car dl))
							   			   (eq? 2 (car x)))))
				          (addch fg_char)
						  (addch bg_char))
				        (set! n (1+ n))
				        (if (< n 6)
				          (clr)))))
	      (clr))
	    (if (not (eq? (cdr y) `()))
	      (loopy (cdr y) (cdr cdigit)))))
	(if (not (eq? (cdr x) `()))
	  (loopx (cdr x) (cdr dl) (cdr ol)))))

(define (gdc-draw-box)
  (attrset boxattr)
  (move yorigin xorigin)
  (vline (ACS_VLINE) 11)
  (hline (ACS_HLINE) 50)
  (addch (ACS_BSSB))
  (move (+ yorigin 11) xorigin)
  (hline (ACS_HLINE) 50)
  (addch (ACS_SSBB))
  (move yorigin (+ xorigin 50))
  (vline (ACS_VLINE) 11)
  (addch (ACS_BBSS))
  (move (+ yorigin 11) (+ xorigin 50))
  (addch (ACS_SBBS))
  (attrset digitattr)
  (move (+ yorigin 5) (+ xorigin 16))
  (addch fg_char)(addch fg_char)
  (move (+ yorigin 7) (+ xorigin 16))
  (addch fg_char)(addch fg_char)
  (move (+ yorigin 5) (+ xorigin 33))
  (addch fg_char)(addch fg_char)
  (move (+ yorigin 7) (+ xorigin 33))
  (addch fg_char)(addch fg_char))
